//////////////////////////////////////////////////////////////////////////////
//   ///////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "_DRAW_2D.h"

//////////////////////////////////////////////////////////////////////////////
int keyExit = 0, preset = 3, HelpTimer = 0; 

//////////////////////////////////////////////////////////////////////////////////////////
inline tfloat sq(tfloat t) { return t * t; }; 
inline tfloat dist(tfloat x1, tfloat y1, tfloat x2, tfloat y2) { return sqrt(sq(x2 - x1) + sq(y2 - y1)); };
inline tfloat force(tfloat r, tfloat x1, tfloat y1, tfloat x2, tfloat y2)
{
  tfloat d = dist(x1, y1, x2, y2); if (d > r) return 0; else return (1 - d / r);
};

//////////////////////////////////////////////////////////////////////////////
#define count 3000

tfloat offsx = -1, offsy = -1, offsc = -0.1, speed = 1.0, radius = 55, reaction = 0.3, falloff = 0.975, border = 0, scale = 1;
int deep = Yes, shdw = Yes;


void presetChange()
{
  preset = (preset + 1) % 4;
  switch(preset)
  {
    case 0: offsx = -1; offsy = -1; offsc = 0.1; break;
    case 1: offsx = -1; offsy = -1; offsc = -0.1; break;
    case 2: offsx = 0; offsy = 0; offsc = 0.1; break;
    case 3: offsx = 0; offsy = 0; offsc = -0.1; break;
  };
};

void keyPressed()                           //  [ ] 
{
  if (II.KeyUp[_Space_]) presetChange();
  if (II.KeyUp[_F1_]) if (HelpTimer) HelpTimer = 0; else HelpTimer = 25 * 10;

  keyExit = (II.KeyUp[_Esc_]);
  II.Reset();
};

//////////////////////////////////////////////////////////////////////////////
class c_circ
{
  public:
  tfloat rad, bord, x, y, dx, dy, xx, yy, ddx, ddy;
  dwRGB c;
  void reset()
  {
     rad  = fRandom(radius * 0.35, radius * 1.15) * scale; bord = rad * border;
     xx = x = fRandom(bord, DrawXL - bord);
     yy = y = fRandom(bord, DrawYL - bord);
     tfloat d = fRandom(3, 5) * speed * scale;
     ddx = 0; dx  = d * RandomSign();
     ddy = 0; dy  = d * RandomSign();
     if (Random(1) == 1) c = RGBis(Random(155, 255), Random(100, 150), 0); else c = RGBis(Random(155, 205), 0, Random(25, 50));
  };
  void regenerate()
  { 
    x += dx; y += dy;
    if ((x > (DrawXL - bord)) || (x < bord)) dx = - dx; else if (Random(37) == 11) dx = - dx;
    if ((y > (DrawYL - bord)) || (y < bord)) dy = - dy; else if (Random(37) == 11) dy = - dy;  
    tfloat d = dist(x, y, xx, yy);
    ddx += (x - xx) / d * reaction;
    ddy += (y - yy) / d * reaction;
    ddx *= falloff; ddy *= falloff;
    xx += ddx; yy += ddy;
  };
  void draw(int i, float r)
  {
    dwRGB cc;
    if (deep) cc = RGBfmultV(c, 0.7 * i / count + 0.4); else cc = c;  
    if (shdw) DrawCircle(xx - offsx * 2.25, yy - offsy * 2.25, r + 4, RGBis(220, 220, 220), modeMUL);
    DrawCircle(xx, yy, r, cc);
    DrawCircle(xx + offsx * r * 0.1, yy + offsy * r * 0.1, 0.8 * r, RGBfmultV(cc, 1 + offsc));
    DrawCircle(xx + offsx * r * 0.25, yy + offsy * r * 0.25, 0.05 * r, RGBis(255, 255, 255));
  };
  void draw(int i, tfloat px1, tfloat py1, tfloat px2, tfloat py2, tfloat px3, tfloat py3, tfloat r)
  {
    r *= scale;
    tfloat f = force(r, xx, yy, px1, py1);
    f += force(r, xx, yy, px2, py2);
    f += force(r, xx, yy, px3, py3);
    if (f > 0) draw(i, rad * f); 
  };
};

/////////////////////////////////////////////////////////////////////////////////////////////////
TextSPR *info;
c_circ circ[count],pcirc[3];

/////////////////////////////////////////////////////////////////////////////////////////////////
void CommonsInit()
{ 
  FC_FpsSet(AUTO);                //    
  FC_DrawPaint = Yes;          //  fps   DrawPaint();
  RandomSeed();
  Draw2D_Init(); ForceActivate(); ScreenLoadWait();
  info = new TextSPR(L"TYPE_EN_RU");
  OnKeyUpDo(keyPressed);       //     
  keyPressed();                //     ...

  scale = (tfloat)DrawXR / 1600; 
  presetChange();
  
  for (int i = 0; i < count; i++) circ[i].reset();
  speed = 2;  
  for (int i = 0; i < 3; i++) pcirc[i].reset();/**/
};


//////////////////////////////////////////////////////////////////////////////////////////////////
void CommonsDone()
{
  delete info;
  Draw2D_Done();
};

//////////////////////////////////////////////////////////////////////////////////////////////////
int infoDrawed = no;
void DrawInfo()
{
  if (!HelpTimer) return;
  HelpTimer--;
  infoDrawed = Yes;
  tRegion r = RegionCenterIs(200, 130);
  DrawRegionStore.Save(r);
  tfRegion fr = RegionTofRegion(r);
  fDrawRoundRect(fr, 20, RGBis(50, 50, 50), modeMUL); 
  info->Place(0, 0, modeADD + fromCC, "Conrol Keys:\n\n\nSpase - Change Preset\n...............................................................................\n\nF1 - hide/show Help\n\nEsc - Exit");
  info->Draw();
};

//////////////////////////////////////////////////////////////////////////////////////////////////
void RestoreInfo()
{
  if (!infoDrawed) return;
  DrawRegionStore.Restore();
  infoDrawed = no;
};

//////////////////////////////////////////////////////////////////////////////////////////////////
void DrawAll()
{
  DrawFill(0);
  for (int i = 0; i < 3; i++) pcirc[i].regenerate(); 
  for (int i = 0; i < count; i++)
  { circ[i].regenerate();
    circ[i].draw(i, pcirc[0].xx, pcirc[0].yy, pcirc[1].xx, pcirc[1].yy, pcirc[2].xx, pcirc[2].yy, 300);
  };  
  DrawInfo(); 
  DrawPaint();
  RestoreInfo();
};

//////////////////////////////////////////////////////////////////////////////
//   //////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void Main()
{
  CommonsInit();                  //   ;  
  
  while (!keyExit) DrawAll();

  CommonsDone();                  //   ;
};
//////////////////////////////////////////////////////////////////////////////
int LetProcess()
{
  //MouseShowCursor = no;
  WinName = L"CG : SMotion";   //    
  VS_CColor = RGBis(0, 0, 0);    //   
  DrawSet(FullScreen);         //   ( )  , FullScreen   
  ProcessFunc = Main;           //    -  
  return on;
};
int Init_LetProcess = LetProcess();

